﻿//***************************************************************************
// (C) 2011 Avery Weigh-Tronix, LLC. 
//***************************************************************************
// TITLE: WTCommScl ActiveX Control Test Program
// DESCR: This is the project source code for the WTCommScl test program. 
//        The project was written in C# using Visual Studio 2008 on a
//        Windows 7 (64-bit) computer.
//        Target platform is x86 so it should run on Win XP (32/64-bit) as
//        well as Win 7 (32/64-bit) that have .NET Framework v2.0 or newer.
//
// (C)Copyright 2011 Avery Weigh-Tronix, LLC.
//
// Contacts:
// Sales:	sales@salterbrecknell.com
//       	1-800-637-0529
//
// Service:	service@salterbrecknell.com
//      	1-800-242-2807
//
// Disclaimer:
// This test/demo program is provided as a courtesy to customers of Avery 
// Weigh-Tronix for their personal use.
//
// NO WARRANTIES: The free-of-charge software is provided “as-is” with no 
// warranties expressed or implied.
// NO LIABILITY: To the maximum extent permitted by applicable law, in no 
// event shall Avery Weigh-Tronix, its suppliers or subsidiaries be liable 
// for any damages whatsoever (including without limitation, damages for 
// loss of business profits, business interruption, loss of business infor-
// mation, or other pecuniary loss) arising out of the installation, use or
// inability to use the software provided, even if Avery Weigh-Tronix has 
// been advised of the possibility of such damages.
// 
// This product was developed for Avery Weigh-Tronix by:
// www.vtxpro.com 
// (C)2011 Vertex Programming -- Santa Rosa, CA
//
// PROGRAMMERS NOTES:
// 1) This app originally used .NET Framework v2.0 but you can use any version.
// 2) This app was originally deployed using ClickOnce technology.
// 3) ClickOnce (on .NET Framework v2.0) does not support placing a shortcut to
//    the application on the desktop at install time. So, a CreateDesktopShortcut()
//    function was added to do this. If you use newer .NET Frameworks that do support
//    desktop icons, then you can omit this function and related namespaces.
// 4) To add a reference to the ActiveX Control:
//    * Right-Click on References in the Solution Explorer.
//    * Click on 'Add Reference...'
//    * Click on the Browse tab.
//    * Browse to find the WTCommScl.OCX file. On Windows 7 it is usually located
//      at c:\Windows\SysWoW64\WTCommScl.OCX but it could be located in the 
//      c:\Program Files (x86)\Avery Weigh-Tronix\.. folder or one of its sub-folders.
//      Therefore, it is easiest to first do a search of your computer to find the
//      exact location of the WTCommScl.OCX file.
//    * Click OK to finish.
// 5) Add the namespace reference at the top of the program as follows:
//    using WTCommScl_OCX;
// 6) To add the ActiveX control to the Toolbox:
//    * Click on the Toolbox tab to open the Toolbox window.
//    * Right-Click on the 'All Windows Forms' section header of the Toolbox window.
//    * Click on 'Choose Items...'
//    * In the Choose Toolbox Items window, click on the COM Components tab.
//    * Scroll down and check the box next to WTCommScl_OCX.WTCommScl
//    * Click OK.
//    The WTCommScl ActiveX Control should now be added to the Toolbox towards the 
//    bottom of the All Windows Forms section.
//    If it does not show up, save all files in your project, exit Visual Studio and
//    then open your project again and it should be in the Toolbox.
// 7) Using the ActiveX Control:
//    ActiveX is older technology but will be supported by Microsoft operating
//    systems until Windows 8 (which will not be able to run ActiveX).
//    The WTCommScl ActiveX Control is not a DLL and it has no constructors. So, it
//    cannot be easily 'instantiated' like other classes. Instead, to use the control
//    you should simply drag and drop the control from the toolbox to your form.
// 8) To prevent runtime error: "Class not registered..."
//    Make sure to set Platform Target: x86 in Project Properties - Build - General
//
// REVISION HISTORY
//
// (v1.0)   10-OCT-12   Developed by:  
//                      www.vtxpro.com for Salter Brecknell (Wayne Willie)
//                      
//                      Updated to support testing of updated WTCommScl ActiveX (v3.0) control
//                      that was modified to provide a more robust interface for stamps.com.
//                      1)Added version (v3.0) to title bar.
//                      2)Added scale weight units of measure to panel.
//

using System;
using System.IO;
using System.Diagnostics;
using System.Windows.Forms;
using System.Threading;

namespace WTCommScl_Tester3
{
    public partial class MAIN : Form
    {
        private short serialPort = 1;

        public MAIN()
        {
            InitializeComponent();
        }

        private void MAIN_Load(object sender, EventArgs e)
        {
            if (UpdateInstalled() == false)
            {
                Application.Exit();
            }

            // Initialize notification
            axWTCommScl1.WtNotification = WTCommScl_OCX.WT_NOTIFY_ENUM.wtEverytime;

            // Put test app major and minor version up on screen
            int dotpos2 = GetIndexOfNthCharInString(Application.ProductVersion, '.', 2);                // version is of the form:  X.X.X.X
            labelTestVer.Text = "(" + Application.ProductVersion.Substring(0, dotpos2) + ")";           // display only (X.X) major.minor version on title bar

            // Put WTCommScl ActiveX Control major and minor version up on screen
            labelActiveXVer.Text = axWTCommScl1.ControlVersion();     // display only (X.X) major.minor version on screen
        }

        private void axWTCommScl1_OnScaleComm(object sender, EventArgs e)
        {
            // Note: ActiveX returns NetWt as a 'Variant' even though document says 'Single'.
            string s = "";

            switch (axWTCommScl1.ScaleCommEvent)
            {
                case wtConst.wtWTCOMM_EV_WEIGHT:
                    textBoxRaw.Text = axWTCommScl1.RawWt;                    // display raw weight string
                    textBoxFormatted.Text = axWTCommScl1.FormattedWt;        // display formatted weight

                    float wt = (float)axWTCommScl1.NetWt;                    // display floating point weight (see note)
                    textBoxNet.Text = string.Format("{0:0.0000}", wt);
                    break;

                case wtConst.wtWTCOMM_EV_STATUS:
                    // Other scale operating mode status handling code here.
                    break;
                case wtConst.wtWTCOMM_EV_DISCONNECT:
                    // Takes about 1 second to detect disconnect based on control internal timer.
                    // Other scale disconnect handling code here.
                    textBoxStatus.Text += "DISCONNECT\r\n";
                    break;

                case wtConst.wtWTCOMM_EV_RECONNECT:
                    textBoxStatus.Text += "CONNECT\r\n";
                    break;
                default:
                    break;
            }

            // Display scale operating mode status dynamically:
            short status = axWTCommScl1.ScaleStatus;
            s = "";

            switch (status)
            {
                case wtConst.wtNORMAL_MODE:
                    s = "NORMAL";                   // 0
                    break;
                case wtConst.wtTEST_MODE:
                    s = "TEST";                     // 1
                    break;
                case wtConst.wtCALIB_MODE:
                    s = "CALIB";                    // 2
                    break;
                case wtConst.wtSHOWING_TARE:
                    s = "TARE";                     // 3
                    break;
                case wtConst.wtSHOWING_LO:
                    s = "LOW";                      // 4
                    break;
                case wtConst.wtSHOWING_ERR:
                    s = "ERROR";                    // 5
                    break;
                case wtConst.wtSHOWING_ERRL:
                    s = "ERRL";                     // 6
                    break;
                case wtConst.wtSHOWING_DASHES:
                    s = "DASHES";                   // 7
                    break;
                case wtConst.wtNOT_USED1:           // 8
                    s = "8";
                    break;
                case wtConst.wtNOT_USED2:           // 9
                    s = "9";
                    break;
                case wtConst.wtNOT_USED3:           // 10
                    s = "10";
                    break;
                case wtConst.wtNOT_USED4:           // 11
                    s = "11";
                    break;
                case wtConst.wtSHOWING_8888:
                    s = "8888";                     // 12
                    break;
                case wtConst.wtSHOWING_TARE_ERR:
                    s = "TARE ERR";                 // 13
                    break;
                case wtConst.wtCALIB_MODE_TARE:
                    s = "CAL TARE";                 // 14
                    break;
                case wtConst.wtSHOWING_CAL:
                    s = "CAL";                      // 15
                    break;
                default:
                    break;
            }
            textBoxScaleOp.Text = s;

            // Display scale raw weight units dynamically:
            short raw_units = axWTCommScl1.RawWtUnits;
            s = "";

            switch (raw_units)
            {
                case wtConst.wtUOM_G:
                    s = "0: grams";
                    break;
                case wtConst.wtUOM_KG:
                    s = "1: kilograms";
                    break;
                case wtConst.wtUOM_LBOZ_DEC:
                    s = "2: lb-oz (dec)";
                    break;
                case wtConst.wtUOM_LBOZ_FRAC:
                    s = "3: lb-oz (fract)";
                    break;
                default:
                    break;
            }
            textBoxRawUnits.Text = s;

            // Display connection status dynamically:
            if (axWTCommScl1.ConnectStatus == wtConst.wtSCALE_ONLINE)
                textBoxConnection.Text = "ONLINE";
            else
            {
                textBoxConnection.Text = "OFFLINE";
                textBoxRaw.Text = "........";
                textBoxFormatted.Text = "........";
                textBoxNet.Text = "........";
                textBoxScaleOp.Text = "........";
            }
        }

        private bool UpdateInstalled()
        {
            // DEVELOPERS:
            // It is recommended that applications using the WTCommScl ActiveX Control first verify that
            // the correct COMINI16.EXE file (for 64-bit compatibility) is located in the Windows directory.

            if (File.Exists(@"c:\Windows\COMINI16.exe"))
            {
                FileVersionInfo file_info = FileVersionInfo.GetVersionInfo(@"c:\Windows\COMINI16.exe");
                int file_ver = file_info.FileMajorPart;

                if (file_ver >= 2011)
                {
                    return (true);
                }
                else
                {
                    string s = "Incorrect version of WTCommScl OCX update file detected.\n";
                    s += "Please run the WTCommScl UPDATE installer then try launching this test program again.\n\n";
                    s += "This program will automatically close now.";
                    MessageBox.Show(s, "Update Required", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return (false);
                }
            }
            else
            {
                string s = "An important file required for WTCommScl OCX was not detected.\n";
                s += "Please run both the WTCommScl installer and also the WTCommScl UPDATE installer, then try launching this test program again.\n\n";
                s += "This program will automatically close now.";
                MessageBox.Show(s, "Update Required", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return (false);
            }
        }

        private void buttonScaleOpen_Click(object sender, EventArgs e)
        {
            // Notes: 
            // 1)Port to C# 2008 converted VB6 Integer parameter to ref to short!?
            // 2)Due to the nature of the fix to allow this 32-bit ActiveX Control
            //   to run on Win 7 (64-bit) machines, the call to the ScaleOpen method
            //   always returns passed (i.e. 0). Therefore, we will not be able to
            //   test its return result or use it as a conditional test value.

            // Set the serial comm port
            serialPort = (short)numericSerialPort.Value;
            axWTCommScl1.CommPort = serialPort;

            // Open serial comm port
            short TrueFalse = -1;                           // In old VB6, False = 0, True = -1
            axWTCommScl1.ScaleOpen(ref TrueFalse);          // see notes
            textBoxStatus.Text += "MANUAL SCALE OPEN\r\n";

            // A little realtime delay (because of note 2)
            Thread.Sleep(1500);

            // Check property status to see if the scale is online 
            if (axWTCommScl1.ConnectStatus == wtConst.wtSCALE_OFFLINE)
            {
                textBoxStatus.Text += "NO DATA FROM SCALE. PLEASE TURN SCALE ON\r\n";
            }
        }

        private void buttonScaleClosed_Click(object sender, EventArgs e)
        {
            // In old VB6, False = 0, True = -1
            short TrueFalse = 0;
            axWTCommScl1.ScaleOpen(ref TrueFalse);

            textBoxConnection.Text = "";
            textBoxRaw.Text = "";
            textBoxFormatted.Text = "";
            textBoxNet.Text = "";
            textBoxScaleOp.Text = "";
            textBoxRawUnits.Text = "";

            textBoxStatus.Text += "MANUAL SCALE CLOSE\r\n";
        }

        // Units:
        private void radioButtonUnits_Click(object sender, EventArgs e)
        {
            if (radioButtonPounds.Checked)
                axWTCommScl1.NetWtUnits = WTCommScl_OCX.WT_UOM_ENUM.wtPounds;

            if (radioButtonKilograms.Checked)
                axWTCommScl1.NetWtUnits = WTCommScl_OCX.WT_UOM_ENUM.wtKilograms;

            if (radioButtonOunces.Checked)
                axWTCommScl1.NetWtUnits = WTCommScl_OCX.WT_UOM_ENUM.wtOunces;

            if (radioButtonGrams.Checked)
                axWTCommScl1.NetWtUnits = WTCommScl_OCX.WT_UOM_ENUM.wtGrams;
        }

        // Notification:
        private void radioButtonEverytime_Click(object sender, EventArgs e)
        {
            axWTCommScl1.WtNotification = WTCommScl_OCX.WT_NOTIFY_ENUM.wtEverytime;
        }

        private void radioButtonWeightChange_CheckedChanged(object sender, EventArgs e)
        {
            axWTCommScl1.WtNotification = WTCommScl_OCX.WT_NOTIFY_ENUM.wtChanged;
        }

        // Serial Port:
        private void numericSerialPort_ValueChanged(object sender, EventArgs e)
        {
            serialPort = (short)numericSerialPort.Value;
        }

        private int GetIndexOfNthCharInString(string s, char t, int n)
        {
            int count = 0;
            for (int i = 0; i < s.Length; i++)
            {
                if (s[i] == t)
                {
                    count++;
                    if (count == n)
                    {
                        return i;
                    }
                }
            }
            return -1;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            textBoxStatus.Text = "";
        }
    }
}
